<?php
require_once __DIR__ . "/../models/ListadoMercadoModel.php";
require_once __DIR__ . "/../models/ProductoModel.php";

class ListadoMercadoController {
    private $model;
    private $productoModel;

    public function __construct($pdo) {
        $this->model = new ListadoMercadoModel($pdo);
        $this->productoModel = new ProductoModel($pdo);
    }

    // Listar todos los productos disponibles
    public function listarProductos() {
        return $this->productoModel->obtenerProductos();
    }

    // Listar todos los listados de mercado
    public function listarListadosMercado($pagina = 1, $por_pagina = 10) {
        $listados = $this->model->obtenerListadosMercado($pagina, $por_pagina);
        // Agregar la cantidad total y el precio ajustado total a cada listado
        foreach ($listados as &$listado) {
            $listado['cantidad_total'] = $this->model->obtenerCantidadTotalPorListado($listado['id_lista_mercado']);
            $listado['precio_ajustado_total'] = $this->model->obtenerPrecioAjustadoTotalPorListado($listado['id_lista_mercado']);
        }
        return $listados;
    }

    // Listar los detalles de un listado específico
    public function listarDetallesLista($id_lista_mercado) {
        if ($id_lista_mercado <= 0) {
            return [];
        }
        return $this->model->obtenerDetallesLista($id_lista_mercado);
    }

    // Actualizar el precio referencial de un producto en la tabla productos
    public function actualizarPrecioReferencial() {
        if ($_SERVER["REQUEST_METHOD"] != "POST" || !isset($_POST["actualizar_precio_referencial"])) {
            return ['success' => false, 'message' => 'Método no permitido.'];
        }

        $id_productos = intval($_POST["id_productos"] ?? 0);
        $precio_referencial = floatval($_POST["precio_referencial"] ?? 0);

        if ($id_productos <= 0) {
            return ['success' => false, 'message' => 'ID de producto inválido.'];
        }
        if ($precio_referencial < 0) {
            return ['success' => false, 'message' => 'El precio referencial no puede ser negativo.'];
        }

        try {
            if ($this->productoModel->actualizarPrecioReferencial($id_productos, $precio_referencial)) {
                return ['success' => true, 'message' => 'Precio referencial actualizado correctamente.'];
            } else {
                return ['success' => false, 'message' => 'No se pudo actualizar el precio referencial.'];
            }
        } catch (Exception $e) {
            error_log("Error al actualizar precio referencial: " . $e->getMessage());
            return ['success' => false, 'message' => 'Error al actualizar el precio referencial.'];
        }
    }

    // Actualizar el precio referencial de un producto en un listado específico
    public function actualizarPrecioReferencialEnListado() {
        if ($_SERVER["REQUEST_METHOD"] != "POST" || !isset($_POST["actualizar_precio_referencial_listado"])) {
            return ['success' => false, 'message' => 'Método no permitido.'];
        }

        $id_lista_mercado_detalle = intval($_POST["id_lista_mercado_detalle"] ?? 0);
        $precio_referencial = floatval($_POST["precio_referencial"] ?? 0);

        if ($id_lista_mercado_detalle <= 0) {
            return ['success' => false, 'message' => 'ID de detalle inválido.'];
        }
        if ($precio_referencial < 0) {
            return ['success' => false, 'message' => 'El precio referencial no puede ser negativo.'];
        }

        try {
            if ($this->model->actualizarPrecioReferencialEnListado($id_lista_mercado_detalle, $precio_referencial)) {
                return ['success' => true, 'message' => 'Precio referencial del listado actualizado correctamente.'];
            } else {
                return ['success' => false, 'message' => 'No se pudo actualizar el precio referencial del listado.'];
            }
        } catch (Exception $e) {
            error_log("Error al actualizar precio referencial del listado: " . $e->getMessage());
            return ['success' => false, 'message' => 'Error al actualizar el precio referencial del listado.'];
        }
    }

    // Crear un nuevo listado de mercado
    public function crearListaMercado() {
        if ($_SERVER["REQUEST_METHOD"] != "POST" || !isset($_POST["crear_lista"])) {
            header("Location: ListadoMercado.php?error=Método no permitido.");
            exit();
        }

        $fecha_lista = trim($_POST["fecha_lista"] ?? '');
        $descripcion = trim($_POST["descripcion"] ?? '') ?: null;

        $date = DateTime::createFromFormat('Y-m-d', $fecha_lista);
        if (!$date || $date->format('Y-m-d') !== $fecha_lista) {
            header("Location: ListadoMercado.php?error=Formato de fecha inválido. Usa YYYY-MM-DD.");
            exit();
        }

        if (empty($fecha_lista)) {
            header("Location: ListadoMercado.php?error=La fecha del listado es obligatoria.");
            exit();
        }

        try {
            $id_lista_mercado = $this->model->crearListaMercado($fecha_lista, $descripcion);
            if ($id_lista_mercado) {
                header("Location: ListadoMercado.php?success=Listado creado correctamente.");
                exit();
            } else {
                header("Location: ListadoMercado.php?error=No se pudo crear el listado.");
                exit();
            }
        } catch (Exception $e) {
            error_log("Error al crear listado: " . $e->getMessage());
            header("Location: ListadoMercado.php?error=Error al crear el listado.");
            exit();
        }
    }

    // Agregar un producto al listado
    public function agregarProductoLista() {
        if ($_SERVER["REQUEST_METHOD"] != "POST" || !isset($_POST["agregar_producto_lista"])) {
            header("Location: ListadoMercado.php?error=Método no permitido.");
            exit();
        }

        $id_lista_mercado = intval($_POST["id_lista_mercado"] ?? 0);
        $id_productos = intval($_POST["id_productos"] ?? 0);
        $cantidad_solicitada = intval($_POST["cantidad_solicitada"] ?? 0);

        if ($id_lista_mercado <= 0) {
            header("Location: ListadoMercado.php?error=Selecciona un listado válido.");
            exit();
        }
        if ($id_productos <= 0) {
            header("Location: ListadoMercado.php?error=Selecciona un producto válido.");
            exit();
        }
        if ($cantidad_solicitada <= 0) {
            header("Location: ListadoMercado.php?error=La cantidad solicitada debe ser mayor que 0.");
            exit();
        }

        // Verificar si el listado y el producto existen
        $listado = $this->model->obtenerListadoPorId($id_lista_mercado);
        if (!$listado) {
            header("Location: ListadoMercado.php?error=El listado no existe.");
            exit();
        }
        $producto = $this->productoModel->obtenerProductoPorId($id_productos);
        if (!$producto) {
            header("Location: ListadoMercado.php?error=El producto no existe.");
            exit();
        }

        // Verificar si el producto ya está en el listado
        if ($this->model->productoExisteEnLista($id_lista_mercado, $id_productos)) {
            header("Location: ListadoMercado.php?error=El producto ya está en el listado. Elimina el producto existente si deseas cambiar la cantidad.");
            exit();
        }

        try {
            if ($this->model->agregarProductoLista(
                $id_lista_mercado,
                $id_productos,
                $cantidad_solicitada,
                $producto['nombreProdu'],
                $producto['categoria'],
                $producto['unidad'],
                $producto['precio_mercado'],
                $producto['precio_referencial'], // Guardamos el precio referencial actual
                $producto['precio_mercado'] * $cantidad_solicitada
            )) {
                header("Location: ListadoMercado.php?id_lista_mercado=$id_lista_mercado&success=Producto agregado al listado correctamente.");
                exit();
            } else {
                header("Location: ListadoMercado.php?error=No se pudo agregar el producto al listado.");
                exit();
            }
        } catch (Exception $e) {
            error_log("Error al agregar producto al listado: " . $e->getMessage());
            header("Location: ListadoMercado.php?error=Error al agregar el producto.");
            exit();
        }
    }

    // Eliminar un producto del listado
    public function eliminarProductoLista() {
        if ($_SERVER["REQUEST_METHOD"] != "POST" || !isset($_POST["eliminar_producto_lista"])) {
            header("Location: ListadoMercado.php?error=Método no permitido.");
            exit();
        }

        $id_lista_mercado_detalle = intval($_POST["id_lista_mercado_detalle"] ?? 0);
        $id_lista_mercado = intval($_POST["id_lista_mercado"] ?? 0);

        if ($id_lista_mercado_detalle <= 0 || $id_lista_mercado <= 0) {
            header("Location: ListadoMercado.php?error=Datos inválidos para eliminar el producto.");
            exit();
        }

        try {
            if ($this->model->eliminarProductoLista($id_lista_mercado_detalle)) {
                header("Location: ListadoMercado.php?id_lista_mercado=$id_lista_mercado&success=Producto eliminado del listado correctamente.");
                exit();
            } else {
                header("Location: ListadoMercado.php?error=No se pudo eliminar el producto del listado.");
                exit();
            }
        } catch (Exception $e) {
            error_log("Error al eliminar producto del listado: " . $e->getMessage());
            header("Location: ListadoMercado.php?error=Error al eliminar el producto.");
            exit();
        }
    }

    // Guardar un listado completo (para ListaRapidaMercado.php)
    public function guardarListaMercado($fecha, $total_general, $detalles) {
        try {
            // Crear el listado principal
            $id_lista_mercado = $this->model->crearListaMercado($fecha, null);
            if (!$id_lista_mercado) {
                throw new Exception("No se pudo crear el listado.");
            }

            // Agregar los detalles
            foreach ($detalles as $detalle) {
                $this->model->agregarProductoLista(
                    $id_lista_mercado,
                    $detalle['id_productos'],
                    $detalle['cantidad'],
                    $detalle['nombreProdu'],
                    $detalle['categoria'],
                    $detalle['unidad'],
                    $detalle['precio_mercado'],
                    $detalle['precio_referencial'], // Guardamos el precio referencial actual
                    $detalle['costo_total']
                );
            }

            // Actualizar el total general
            $this->model->actualizarTotalGeneral($id_lista_mercado, $total_general);
            return $id_lista_mercado;
        } catch (Exception $e) {
            error_log("Error al guardar lista de mercado: " . $e->getMessage());
            throw $e;
        }
    }
}