<?php
require_once __DIR__ . "/../models/ReservaGeneralModel.php";
require_once __DIR__ . "/../models/GuiaModel.php";
require_once __DIR__ . "/../models/AgenciaModel.php";

class ReservaController {
    private $model;
    private $guiaModel;
    private $agenciaModel;

    public function __construct($pdo) {
        $this->model = new ReservaGeneralModel($pdo);
        $this->guiaModel = new GuiaModel($pdo);
        $this->agenciaModel = new AgenciaModel($pdo);
    }

    // Listar todas las reservas con sus relaciones (guía y agencia), con soporte para filtros
    public function listarReservasConRelaciones($fechaFiltro = null, $idGuiaFiltro = null, $idAgenciaFiltro = null) {
        $reservas = $this->model->obtenerReservas($fechaFiltro, $idGuiaFiltro, $idAgenciaFiltro);
        return [
            'reservas' => $reservas,
            'guias' => $this->guiaModel->obtenerGuias(),
            'agencias' => $this->agenciaModel->obtenerAgencias()
        ];
    }

    // Listar guías para el formulario
    public function listarGuias() {
        return $this->guiaModel->obtenerGuias();
    }

    // Listar agencias para el formulario
    public function listarAgencias() {
        return $this->agenciaModel->obtenerAgencias();
    }

    // Agregar una nueva reserva
    public function agregarReserva() {
        if ($_SERVER["REQUEST_METHOD"] == "POST" && isset($_POST["agregar_reserva"])) {
            $fecha = trim($_POST["fecha_mes_y_anio"]);
            $id_guia = intval($_POST["id_guia"]);
            $id_agencia = intval($_POST["id_agencia"]);
            $numero_pasajeros = intval($_POST["numero_pasajeros"]);
            $firma = trim($_POST["firma"]) ?: null;
            $observaciones = trim($_POST["observaciones"]) ?: null;
            $adelanto = isset($_POST["adelanto"]) ? floatval($_POST["adelanto"]) : null;
            $metodo_pago = trim($_POST["metodo_pago"]) ?: null;
            $estado = trim($_POST["estado"]);

            // Validaciones
            if (empty($fecha)) {
                header("Location: reservacreate.php?error=La fecha es obligatoria.");
                exit();
            }
            if (strtotime($fecha) < strtotime(date('Y-m-d'))) {
                header("Location: reservacreate.php?error=La fecha no puede ser anterior a hoy.");
                exit();
            }
            if ($id_guia <= 0) {
                header("Location: reservacreate.php?error=Selecciona un guía válido.");
                exit();
            }
            if ($id_agencia <= 0) {
                header("Location: reservacreate.php?error=Selecciona una agencia válida.");
                exit();
            }
            if ($numero_pasajeros <= 0 || $numero_pasajeros > 100) {
                header("Location: reservacreate.php?error=El número de pasajeros debe estar entre 1 y 100.");
                exit();
            }
            if (empty($firma)) {
                header("Location: reservacreate.php?error=La firma no puede estar vacía.");
                exit();
            }
            if ($adelanto === null || $adelanto < 0) {
                header("Location: reservacreate.php?error=El adelanto no puede ser negativo.");
                exit();
            }
            if (empty($metodo_pago)) {
                header("Location: reservacreate.php?error=El método de pago no puede estar vacío.");
                exit();
            }
            if (!in_array($estado, ['pagado', 'sin pagar'])) {
                header("Location: reservacreate.php?error=El estado debe ser 'pagado' o 'sin pagar'.");
                exit();
            }

            // Validar formato de fecha (YYYY-MM-DD)
            $date = DateTime::createFromFormat('Y-m-d', $fecha);
            if (!$date || $date->format('Y-m-d') !== $fecha) {
                header("Location: reservacreate.php?error=La fecha debe estar en formato YYYY-MM-DD.");
                exit();
            }

            // Llamar al modelo para agregar la reserva
            if ($this->model->agregarReserva($fecha, $id_guia, $id_agencia, $numero_pasajeros, $firma, $observaciones, $adelanto, $metodo_pago, $estado)) {
                header("Location: reservacreate.php?success=Reserva agregada correctamente.");
                exit();
            } else {
                header("Location: reservacreate.php?error=No se pudo agregar la reserva.");
                exit();
            }
        }
    }

    // Editar una reserva
    public function editarReserva() {
        if ($_SERVER["REQUEST_METHOD"] == "POST" && isset($_POST["editar_reserva"])) {
            // Depuración: Verificar que los datos se están recibiendo
            error_log("Datos recibidos en editarReserva: " . print_r($_POST, true));

            $id_reservaciongeneral = isset($_POST["id_reservaciongeneral"]) ? intval($_POST["id_reservaciongeneral"]) : 0;
            $fecha = trim($_POST["fecha_mes_y_anio"]);
            $id_guia = intval($_POST["id_guia"]);
            $id_agencia = intval($_POST["id_agencia"]);
            $numero_pasajeros = intval($_POST["numero_pasajeros"]);
            $firma = trim($_POST["firma"]) ?: null;
            $observaciones = trim($_POST["observaciones"]) ?: null;
            $adelanto = isset($_POST["adelanto"]) ? floatval($_POST["adelanto"]) : null;
            $metodo_pago = trim($_POST["metodo_pago"]) ?: null;
            $estado = trim($_POST["estado"]);

            // Depuración: Verificar los valores recibidos
            error_log("Valores procesados: id_reservaciongeneral=$id_reservaciongeneral, fecha=$fecha, id_guia=$id_guia, id_agencia=$id_agencia, numero_pasajeros=$numero_pasajeros, firma=$firma, observaciones=$observaciones, adelanto=$adelanto, metodo_pago=$metodo_pago, estado=$estado");

            // Validaciones
            if ($id_reservaciongeneral <= 0) {
                error_log("Error: ID de reserva inválido.");
                header("Location: reservaedit.php?error=ID de reserva inválido.");
                exit();
            }
            if (empty($fecha)) {
                error_log("Error: La fecha es obligatoria.");
                header("Location: reservaedit.php?error=La fecha es obligatoria.");
                exit();
            }
            if (strtotime($fecha) < strtotime(date('Y-m-d'))) {
                error_log("Error: La fecha no puede ser anterior a hoy.");
                header("Location: reservaedit.php?error=La fecha no puede ser anterior a hoy.");
                exit();
            }
            if ($id_guia <= 0) {
                error_log("Error: Selecciona un guía válido.");
                header("Location: reservaedit.php?error=Selecciona un guía válido.");
                exit();
            }
            if ($id_agencia <= 0) {
                error_log("Error: Selecciona una agencia válida.");
                header("Location: reservaedit.php?error=Selecciona una agencia válida.");
                exit();
            }
            if ($numero_pasajeros <= 0 || $numero_pasajeros > 100) {
                error_log("Error: El número de pasajeros debe estar entre 1 y 100.");
                header("Location: reservaedit.php?error=El número de pasajeros debe estar entre 1 y 100.");
                exit();
            }
            if (empty($firma)) {
                error_log("Error: La firma no puede estar vacía.");
                header("Location: reservaedit.php?error=La firma no puede estar vacía.");
                exit();
            }
            if ($adelanto === null || $adelanto < 0) {
                error_log("Error: El adelanto no puede ser negativo.");
                header("Location: reservaedit.php?error=El adelanto no puede ser negativo.");
                exit();
            }
            if (empty($metodo_pago)) {
                error_log("Error: El método de pago no puede estar vacío.");
                header("Location: reservaedit.php?error=El método de pago no puede estar vacío.");
                exit();
            }
            if (!in_array($estado, ['pagado', 'sin pagar'])) {
                error_log("Error: El estado debe ser 'pagado' o 'sin pagar'.");
                header("Location: reservaedit.php?error=El estado debe ser 'pagado' o 'sin pagar'.");
                exit();
            }

            // Validar formato de fecha (YYYY-MM-DD)
            $date = DateTime::createFromFormat('Y-m-d', $fecha);
            if (!$date || $date->format('Y-m-d') !== $fecha) {
                error_log("Error: La fecha debe estar en formato YYYY-MM-DD.");
                header("Location: reservaedit.php?error=La fecha debe estar en formato YYYY-MM-DD.");
                exit();
            }

            // Llamar al modelo para editar la reserva
            $result = $this->model->editarReserva($id_reservaciongeneral, $fecha, $id_guia, $id_agencia, $numero_pasajeros, $firma, $observaciones, $adelanto, $metodo_pago, $estado);
            if ($result) {
                error_log("Éxito: Reserva actualizada correctamente.");
                header("Location: reservacreate.php?success=Reserva actualizada correctamente.");
                exit();
            } else {
                error_log("Error: No se pudo actualizar la reserva en el modelo.");
                header("Location: reservaedit.php?error=No se pudo actualizar la reserva.");
                exit();
            }
        } else {
            error_log("Error: Solicitud no válida para editarReserva.");
            header("Location: reservaedit.php?error=Solicitud no válida.");
            exit();
        }
    }

    // Eliminar una reserva
    public function eliminarReserva() {
        if ($_SERVER["REQUEST_METHOD"] == "POST" && isset($_POST["eliminar_reserva"])) {
            $id_reservaciongeneral = intval($_POST["id_reservaciongeneral"]);

            if ($this->model->eliminarReserva($id_reservaciongeneral)) {
                header("Location: reservacreate.php?success=Reserva eliminada correctamente.");
                exit();
            } else {
                header("Location: reservacreate.php?error=No se pudo eliminar la reserva.");
                exit();
            }
        }
    }
}