<?php
require_once __DIR__ . "/../config/config.php";

class ListadoMercadoModel {
    private $pdo;

    public function __construct($pdo) {
        $this->pdo = $pdo;
    }

    // Obtener todos los listados de mercado con paginación
    public function obtenerListadosMercado($pagina = 1, $por_pagina = 10) {
        try {
            $offset = ($pagina - 1) * $por_pagina;
            $stmt = $this->pdo->prepare("
                SELECT * FROM lista_mercado 
                ORDER BY fecha_lista DESC 
                LIMIT :offset, :por_pagina
            ");
            $stmt->bindValue(':offset', $offset, PDO::PARAM_INT);
            $stmt->bindValue(':por_pagina', $por_pagina, PDO::PARAM_INT);
            $stmt->execute();
            return $stmt->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            throw new Exception("Error al obtener los listados de mercado: " . $e->getMessage());
        }
    }

    // Obtener la cantidad total de productos por listado
    public function obtenerCantidadTotalPorListado($id_lista_mercado) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT SUM(cantidad_solicitada) AS cantidad_total
                FROM lista_mercado_detalle
                WHERE id_lista_mercado = :id_lista_mercado
            ");
            $stmt->execute([':id_lista_mercado' => $id_lista_mercado]);
            $result = $stmt->fetch(PDO::FETCH_ASSOC);
            return $result['cantidad_total'] ?? 0;
        } catch (PDOException $e) {
            throw new Exception("Error al obtener la cantidad total del listado: " . $e->getMessage());
        }
    }

    // Calcular el precio ajustado total (cantidad_solicitada * precio_referencial)
    public function obtenerPrecioAjustadoTotalPorListado($id_lista_mercado) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT SUM(lmd.cantidad_solicitada * lmd.precio_referencial) AS precio_ajustado_total
                FROM lista_mercado_detalle lmd
                WHERE lmd.id_lista_mercado = :id_lista_mercado
            ");
            $stmt->execute([':id_lista_mercado' => $id_lista_mercado]);
            $result = $stmt->fetch(PDO::FETCH_ASSOC);
            $precio_ajustado_total = $result['precio_ajustado_total'] ?? 0;
            error_log("Precio Ajustado Total para listado $id_lista_mercado: $precio_ajustado_total");
            return $precio_ajustado_total;
        } catch (PDOException $e) {
            throw new Exception("Error al calcular el precio ajustado total: " . $e->getMessage());
        }
    }

    // Obtener un listado específico por ID
    public function obtenerListadoPorId($id_lista_mercado) {
        try {
            $stmt = $this->pdo->prepare("SELECT * FROM lista_mercado WHERE id_lista_mercado = :id");
            $stmt->execute([':id' => $id_lista_mercado]);
            return $stmt->fetch(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            throw new Exception("Error al obtener el listado: " . $e->getMessage());
        }
    }

    // Obtener los detalles de un listado específico
    public function obtenerDetallesLista($id_lista_mercado) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT 
                    lmd.id_lista_mercado_detalle,
                    lmd.id_lista_mercado,
                    lmd.id_productos,
                    lmd.cantidad_solicitada,
                    lmd.nombre_producto,
                    lmd.categoria,
                    lmd.unidad,
                    lmd.precio_mercado,
                    lmd.precio_referencial,
                    lmd.costo_total,
                    p.nombreProdu,
                    p.categoria AS categoria_producto,
                    p.unidad AS unidad_producto,
                    p.precio_abarrote,
                    p.precio_mercado AS precio_mercado_producto,
                    p.precio_referencial AS precio_referencial_producto
                FROM lista_mercado_detalle lmd
                JOIN productos p ON lmd.id_productos = p.id_productos
                WHERE lmd.id_lista_mercado = :id_lista_mercado
            ");
            $stmt->execute([":id_lista_mercado" => $id_lista_mercado]);
            return $stmt->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            throw new Exception("Error al obtener los detalles del listado: " . $e->getMessage());
        }
    }

    // Verificar si un producto ya existe en un listado
    public function productoExisteEnLista($id_lista_mercado, $id_productos) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT COUNT(*) 
                FROM lista_mercado_detalle 
                WHERE id_lista_mercado = :id_lista AND id_productos = :id_producto
            ");
            $stmt->execute([
                ':id_lista' => $id_lista_mercado,
                ':id_producto' => $id_productos
            ]);
            return $stmt->fetchColumn() > 0;
        } catch (PDOException $e) {
            throw new Exception("Error al verificar si el producto existe en el listado: " . $e->getMessage());
        }
    }

    // Crear un nuevo listado de mercado
    public function crearListaMercado($fecha, $descripcion) {
        try {
            $stmt = $this->pdo->prepare("
                INSERT INTO lista_mercado (fecha_lista, descripcion) 
                VALUES (:fecha_lista, :descripcion)
            ");
            $stmt->execute([
                ":fecha_lista" => $fecha,
                ":descripcion" => $descripcion
            ]);
            return $this->pdo->lastInsertId();
        } catch (PDOException $e) {
            throw new Exception("Error al crear el listado de mercado: " . $e->getMessage());
        }
    }

    // Agregar un producto al listado
    public function agregarProductoLista($id_lista_mercado, $id_productos, $cantidad_solicitada, $nombre_producto = null, $categoria = null, $unidad = null, $precio_mercado = null, $precio_referencial = null, $costo_total = null) {
        try {
            $stmt = $this->pdo->prepare("
                INSERT INTO lista_mercado_detalle (
                    id_lista_mercado, 
                    id_productos, 
                    cantidad_solicitada, 
                    nombre_producto, 
                    categoria, 
                    unidad, 
                    precio_mercado, 
                    precio_referencial, 
                    costo_total
                ) 
                VALUES (
                    :id_lista_mercado, 
                    :id_productos, 
                    :cantidad_solicitada, 
                    :nombre_producto, 
                    :categoria, 
                    :unidad, 
                    :precio_mercado, 
                    :precio_referencial, 
                    :costo_total
                )
            ");
            return $stmt->execute([
                ":id_lista_mercado" => $id_lista_mercado,
                ":id_productos" => $id_productos,
                ":cantidad_solicitada" => $cantidad_solicitada,
                ":nombre_producto" => $nombre_producto,
                ":categoria" => $categoria,
                ":unidad" => $unidad,
                ":precio_mercado" => $precio_mercado,
                ":precio_referencial" => $precio_referencial,
                ":costo_total" => $costo_total
            ]);
        } catch (PDOException $e) {
            throw new Exception("Error al agregar el producto al listado: " . $e->getMessage());
        }
    }

    // Actualizar el precio referencial de un producto en un listado específico
    public function actualizarPrecioReferencialEnListado($id_lista_mercado_detalle, $precio_referencial) {
        if ($precio_referencial < 0) {
            throw new Exception("El precio referencial no puede ser negativo.");
        }

        try {
            $stmt = $this->pdo->prepare("
                UPDATE lista_mercado_detalle 
                SET precio_referencial = :precio_referencial 
                WHERE id_lista_mercado_detalle = :id
            ");
            $result = $stmt->execute([
                ':precio_referencial' => $precio_referencial,
                ':id' => $id_lista_mercado_detalle
            ]);
            if ($result) {
                error_log("Precio referencial del detalle con ID $id_lista_mercado_detalle actualizado a $precio_referencial.");
            }
            return $result;
        } catch (PDOException $e) {
            error_log("Error al actualizar el precio referencial del detalle con ID $id_lista_mercado_detalle: " . $e->getMessage());
            throw new Exception("Error al actualizar el precio referencial: " . $e->getMessage());
        }
    }

    // Actualizar el total general de un listado
    public function actualizarTotalGeneral($id_lista_mercado, $total_general) {
        try {
            $stmt = $this->pdo->prepare("
                UPDATE lista_mercado 
                SET total_general = :total 
                WHERE id_lista_mercado = :id
            ");
            return $stmt->execute([
                ':total' => $total_general,
                ':id' => $id_lista_mercado
            ]);
        } catch (PDOException $e) {
            throw new Exception("Error al actualizar el total general: " . $e->getMessage());
        }
    }

    // Eliminar un producto del listado
    public function eliminarProductoLista($id_lista_mercado_detalle) {
        try {
            $stmt = $this->pdo->prepare("
                DELETE FROM lista_mercado_detalle 
                WHERE id_lista_mercado_detalle = :id_lista_mercado_detalle
            ");
            return $stmt->execute([":id_lista_mercado_detalle" => $id_lista_mercado_detalle]);
        } catch (PDOException $e) {
            throw new Exception("Error al eliminar el producto del listado: " . $e->getMessage());
        }
    }
}