<?php
require_once __DIR__ . "/../config/config.php";

class MesaModel {
    private $pdo;

    public function __construct($pdo) {
        $this->pdo = $pdo;
    }

    // Generar mesas para un día específico
    public function generarMesas($fecha) {
        $stmt = $this->pdo->prepare("
            INSERT INTO mesa (id_reservaciongeneral, fecha, numero_pasajeros, id_guia, id_agencia, estado_atencion)
            SELECT 
                id_reservaciongeneral,
                fecha_mes_y_año,
                numero_pasajeros,
                id_guia,
                id_agencia,
                'Sin atender' AS estado_atencion
            FROM reserva_general
            WHERE fecha_mes_y_año = :fecha
            AND NOT EXISTS (
                SELECT 1 
                FROM mesa 
                WHERE mesa.id_reservaciongeneral = reserva_general.id_reservaciongeneral
            )
        ");
        return $stmt->execute([":fecha" => $fecha]);
    }

    // Obtener mesas para un día
    public function obtenerMesas($fecha) {
        $stmt = $this->pdo->prepare("
            SELECT 
                m.id_mesa,
                m.fecha,
                m.numero_pasajeros,
                g.nombreGuia,
                a.nombreAgencia,
                m.estado_atencion
            FROM mesa m
            JOIN guia g ON m.id_guia = g.id_guia
            JOIN agencia a ON m.id_agencia = a.id_agencia
            WHERE m.fecha = :fecha
            ORDER BY m.id_mesa
        ");
        $stmt->execute([":fecha" => $fecha]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }

    // Actualizar estado de una mesa
    public function actualizarEstadoMesa($id_mesa, $estado_atencion) {
        if (!in_array($estado_atencion, ['Atendido', 'Sin atender', 'Rechazado'])) {
            return false;
        }
        $stmt = $this->pdo->prepare("
            UPDATE mesa 
            SET estado_atencion = :estado_atencion
            WHERE id_mesa = :id_mesa
        ");
        return $stmt->execute([
            ":estado_atencion" => $estado_atencion,
            ":id_mesa" => $id_mesa
        ]);
    }

    // Obtener productos disponibles
    public function obtenerProductos() {
        $stmt = $this->pdo->prepare("
            SELECT id_productos, nombreProdu, precio_mercado
            FROM productos
            ORDER BY nombreProdu ASC
        ");
        $stmt->execute();
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }

    // Agregar un producto a una mesa
    public function agregarProductoMesa($id_mesa, $id_producto, $cantidad) {
        // Obtener el precio del producto
        $stmt = $this->pdo->prepare("SELECT precio_mercado FROM productos WHERE id_productos = :id_producto");
        $stmt->execute([":id_producto" => $id_producto]);
        $producto = $stmt->fetch(PDO::FETCH_ASSOC);
        if (!$producto) {
            return false;
        }
        $precio_unitario = $producto['precio_mercado'];

        // Generar código único (por ejemplo, MESA1-PROD2-20250417)
        $fecha = date('Ymd');
        $codigo_unico = "MESA{$id_mesa}-PROD{$id_producto}-{$fecha}" . uniqid();

        // Insertar el producto
        $stmt = $this->pdo->prepare("
            INSERT INTO mesa_productos (id_mesa, codigo_unico, id_producto, cantidad, estado_pago, check_verificado, precio_unitario)
            VALUES (:id_mesa, :codigo_unico, :id_producto, :cantidad, 'sin pagar', FALSE, :precio_unitario)
        ");
        return $stmt->execute([
            ":id_mesa" => $id_mesa,
            ":codigo_unico" => $codigo_unico,
            ":id_producto" => $id_producto,
            ":cantidad" => $cantidad,
            ":precio_unitario" => $precio_unitario
        ]);
    }

    // Obtener productos de una mesa
    public function obtenerProductosMesa($id_mesa) {
        $stmt = $this->pdo->prepare("
            SELECT 
                mp.id_mesa_producto,
                mp.codigo_unico,
                p.nombreProdu,
                mp.cantidad,
                mp.estado_pago,
                mp.check_verificado,
                mp.precio_unitario,
                mp.costo_total
            FROM mesa_productos mp
            JOIN productos p ON mp.id_producto = p.id_productos
            WHERE mp.id_mesa = :id_mesa
            ORDER BY mp.creado_en ASC
        ");
        $stmt->execute([":id_mesa" => $id_mesa]);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }

    // Actualizar estado de pago de un producto
    public function actualizarEstadoPagoProducto($id_mesa_producto, $estado_pago, $check_verificado) {
        $stmt = $this->pdo->prepare("
            UPDATE mesa_productos 
            SET estado_pago = :estado_pago, check_verificado = :check_verificado
            WHERE id_mesa_producto = :id_mesa_producto
        ");
        return $stmt->execute([
            ":estado_pago" => $estado_pago,
            ":check_verificado" => $check_verificado ? 1 : 0,
            ":id_mesa_producto" => $id_mesa_producto
        ]);
    }
}