<?php
require_once __DIR__ . "/../config/config.php";

class ProductoModel {
    private $pdo;

    public function __construct($pdo) {
        $this->pdo = $pdo;
    }

    // Obtener todos los productos
    public function obtenerProductos() {
        try {
            $stmt = $this->pdo->prepare("
                SELECT 
                    id_productos, 
                    nombreProdu, 
                    cantidad, 
                    fecha_pedido, 
                    unidad, 
                    precio_abarrote, 
                    precio_mercado, 
                    precio_referencial, 
                    categoria 
                FROM productos 
                ORDER BY nombreProdu ASC
            ");
            $stmt->execute();
            return $stmt->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            error_log("Error al obtener los productos: " . $e->getMessage());
            throw new Exception("Error al obtener los productos: " . $e->getMessage());
        }
    }

    // Obtener un producto por ID
    public function obtenerProductoPorId($id_productos) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT 
                    id_productos, 
                    nombreProdu, 
                    cantidad, 
                    fecha_pedido, 
                    unidad, 
                    precio_abarrote, 
                    precio_mercado, 
                    precio_referencial, 
                    categoria 
                FROM productos 
                WHERE id_productos = :id
            ");
            $stmt->execute([':id' => $id_productos]);
            $producto = $stmt->fetch(PDO::FETCH_ASSOC);
            if (!$producto) {
                throw new Exception("Producto con ID $id_productos no encontrado.");
            }
            return $producto;
        } catch (PDOException $e) {
            error_log("Error al obtener el producto con ID $id_productos: " . $e->getMessage());
            throw new Exception("Error al obtener el producto: " . $e->getMessage());
        }
    }

    // Agregar un producto
    public function agregarProducto($nombre, $cantidad, $fecha_pedido, $unidad, $precio_abarrote, $precio_mercado, $precio_referencial, $categoria) {
        // Validaciones
        if (empty($nombre)) {
            throw new Exception("El nombre del producto es obligatorio.");
        }
        if ($cantidad < 0) {
            throw new Exception("La cantidad no puede ser negativa.");
        }
        if (!DateTime::createFromFormat('Y-m-d', $fecha_pedido)) {
            throw new Exception("Formato de fecha inválido. Use YYYY-MM-DD.");
        }
        if ($precio_abarrote < 0 || $precio_mercado < 0 || $precio_referencial < 0) {
            throw new Exception("Los precios no pueden ser negativos.");
        }

        try {
            $stmt = $this->pdo->prepare("
                INSERT INTO productos (nombreProdu, cantidad, fecha_pedido, unidad, precio_abarrote, precio_mercado, precio_referencial, categoria) 
                VALUES (:nombre, :cantidad, :fecha_pedido, :unidad, :precio_abarrote, :precio_mercado, :precio_referencial, :categoria)
            ");
            $result = $stmt->execute([
                ":nombre" => $nombre,
                ":cantidad" => $cantidad,
                ":fecha_pedido" => $fecha_pedido,
                ":unidad" => $unidad,
                ":precio_abarrote" => $precio_abarrote,
                ":precio_mercado" => $precio_mercado,
                ":precio_referencial" => $precio_referencial,
                ":categoria" => $categoria
            ]);
            if ($result) {
                error_log("Producto '$nombre' agregado correctamente.");
            }
            return $result;
        } catch (PDOException $e) {
            error_log("Error al agregar el producto '$nombre': " . $e->getMessage());
            throw new Exception("Error al agregar el producto: " . $e->getMessage());
        }
    }

    // Editar un producto
    public function editarProducto($id_productos, $nombre, $cantidad, $fecha_pedido, $unidad, $precio_abarrote, $precio_mercado, $precio_referencial, $categoria) {
        // Validaciones
        if (empty($nombre)) {
            throw new Exception("El nombre del producto es obligatorio.");
        }
        if ($cantidad < 0) {
            throw new Exception("La cantidad no puede ser negativa.");
        }
        if (!DateTime::createFromFormat('Y-m-d', $fecha_pedido)) {
            throw new Exception("Formato de fecha inválido. Use YYYY-MM-DD.");
        }
        if ($precio_abarrote < 0 || $precio_mercado < 0 || $precio_referencial < 0) {
            throw new Exception("Los precios no pueden ser negativos.");
        }

        try {
            $stmt = $this->pdo->prepare("
                UPDATE productos 
                SET 
                    nombreProdu = :nombre, 
                    cantidad = :cantidad, 
                    fecha_pedido = :fecha_pedido, 
                    unidad = :unidad, 
                    precio_abarrote = :precio_abarrote, 
                    precio_mercado = :precio_mercado, 
                    precio_referencial = :precio_referencial, 
                    categoria = :categoria 
                WHERE id_productos = :id_productos
            ");
            $result = $stmt->execute([
                ":nombre" => $nombre,
                ":cantidad" => $cantidad,
                ":fecha_pedido" => $fecha_pedido,
                ":unidad" => $unidad,
                ":precio_abarrote" => $precio_abarrote,
                ":precio_mercado" => $precio_mercado,
                ":precio_referencial" => $precio_referencial,
                ":categoria" => $categoria,
                ":id_productos" => $id_productos
            ]);
            if ($result) {
                error_log("Producto con ID $id_productos editado correctamente.");
            }
            return $result;
        } catch (PDOException $e) {
            error_log("Error al editar el producto con ID $id_productos: " . $e->getMessage());
            throw new Exception("Error al editar el producto: " . $e->getMessage());
        }
    }

    // Actualizar el precio referencial de un producto
    public function actualizarPrecioReferencial($id_productos, $precio_referencial) {
        if ($precio_referencial < 0) {
            throw new Exception("El precio referencial no puede ser negativo.");
        }

        try {
            $stmt = $this->pdo->prepare("
                UPDATE productos 
                SET precio_referencial = :precio_referencial 
                WHERE id_productos = :id
            ");
            $result = $stmt->execute([
                ':precio_referencial' => $precio_referencial,
                ':id' => $id_productos
            ]);
            if ($result) {
                error_log("Precio referencial del producto con ID $id_productos actualizado a $precio_referencial.");
            }
            return $result;
        } catch (PDOException $e) {
            error_log("Error al actualizar el precio referencial del producto con ID $id_productos: " . $e->getMessage());
            throw new Exception("Error al actualizar el precio referencial: " . $e->getMessage());
        }
    }

    // Eliminar un producto
    public function eliminarProducto($id_productos) {
        try {
            $stmt = $this->pdo->prepare("DELETE FROM productos WHERE id_productos = :id_productos");
            $result = $stmt->execute([":id_productos" => $id_productos]);
            if ($result) {
                error_log("Producto con ID $id_productos eliminado correctamente.");
            }
            return $result;
        } catch (PDOException $e) {
            error_log("Error al eliminar el producto con ID $id_productos: " . $e->getMessage());
            throw new Exception("Error al eliminar el producto: " . $e->getMessage());
        }
    }

    // Buscar productos por categoría
    public function buscarProductosPorCategoria($categoria) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT 
                    id_productos, 
                    nombreProdu, 
                    cantidad, 
                    fecha_pedido, 
                    unidad, 
                    precio_abarrote, 
                    precio_mercado, 
                    precio_referencial, 
                    categoria 
                FROM productos 
                WHERE categoria = :categoria 
                ORDER BY nombreProdu ASC
            ");
            $stmt->execute([':categoria' => $categoria]);
            return $stmt->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            error_log("Error al buscar productos por categoría '$categoria': " . $e->getMessage());
            throw new Exception("Error al buscar productos por categoría: " . $e->getMessage());
        }
    }

    // Buscar productos por nombre (búsqueda parcial)
    public function buscarProductosPorNombre($nombre) {
        try {
            $stmt = $this->pdo->prepare("
                SELECT 
                    id_productos, 
                    nombreProdu, 
                    cantidad, 
                    fecha_pedido, 
                    unidad, 
                    precio_abarrote, 
                    precio_mercado, 
                    precio_referencial, 
                    categoria 
                FROM productos 
                WHERE nombreProdu LIKE :nombre 
                ORDER BY nombreProdu ASC
            ");
            $stmt->execute([':nombre' => "%$nombre%"]);
            return $stmt->fetchAll(PDO::FETCH_ASSOC);
        } catch (PDOException $e) {
            error_log("Error al buscar productos por nombre '$nombre': " . $e->getMessage());
            throw new Exception("Error al buscar productos por nombre: " . $e->getMessage());
        }
    }
}