<?php
require_once __DIR__ . "/../config/config.php";

class ReservaGeneralModel {
    private $pdo;

    public function __construct($pdo) {
        $this->pdo = $pdo;
    }

    // Obtener todas las reservas con sus relaciones, con soporte para filtros
    public function obtenerReservas($fechaInicio = null, $fechaFin = null, $idGuiaFiltro = null, $idAgenciaFiltro = null) {
        $query = "
            SELECT r.*, g.nombreGuia, a.nombreAgencia, a.precio AS precio_agencia
            FROM reserva_general r
            JOIN guia g ON r.id_guia = g.id_guia
            JOIN agencia a ON r.id_agencia = a.id_agencia
            WHERE 1=1
        ";
        $params = [];

        // Filtro por rango de fechas
        if ($fechaInicio) {
            $query .= " AND r.fecha_mes_y_año >= :fechaInicio";
            $params[':fechaInicio'] = $fechaInicio;
        }
        if ($fechaFin) {
            $query .= " AND r.fecha_mes_y_año <= :fechaFin";
            $params[':fechaFin'] = $fechaFin;
        }

        // Filtro por guía
        if ($idGuiaFiltro) {
            $query .= " AND r.id_guia = :id_guia";
            $params[':id_guia'] = $idGuiaFiltro;
        }

        // Filtro por agencia
        if ($idAgenciaFiltro) {
            $query .= " AND r.id_agencia = :id_agencia";
            $params[':id_agencia'] = $idAgenciaFiltro;
        }

        $query .= " ORDER BY r.fecha_mes_y_año DESC";

        $stmt = $this->pdo->prepare($query);
        $stmt->execute($params);
        $reservas = $stmt->fetchAll(PDO::FETCH_ASSOC);

        // Calcular el campo 'totalidad' ((precio_agencia * numero_pasajeros) - adelanto)
        foreach ($reservas as &$reserva) {
            if ($reserva['precio_agencia'] !== null && $reserva['numero_pasajeros'] !== null && $reserva['adelanto'] !== null) {
                $reserva['totalidad'] = ($reserva['precio_agencia'] * $reserva['numero_pasajeros']) - $reserva['adelanto'];
                $reserva['totalidad'] = max(0, $reserva['totalidad']);
                $this->actualizarTotalidad($reserva['id_reservaciongeneral'], $reserva['totalidad']);
            } else {
                $reserva['totalidad'] = null;
            }
        }

        return $reservas;
    }

    // Actualizar el campo 'totalidad' en la base de datos
    private function actualizarTotalidad($id_reservaciongeneral, $totalidad) {
        $stmt = $this->pdo->prepare("UPDATE reserva_general SET totalidad = :totalidad WHERE id_reservaciongeneral = :id_reservaciongeneral");
        return $stmt->execute([
            ":totalidad" => $totalidad,
            ":id_reservaciongeneral" => $id_reservaciongeneral
        ]);
    }

    // Obtener una reserva por ID (para calcular el total después de agregar/editar)
    public function obtenerReservaPorId($id_reservaciongeneral) {
        $stmt = $this->pdo->prepare("
            SELECT r.*, a.precio AS precio_agencia
            FROM reserva_general r
            LEFT JOIN agencia a ON r.id_agencia = a.id_agencia
            WHERE r.id_reservaciongeneral = :id_reservaciongeneral
        ");
        $stmt->execute([":id_reservaciongeneral" => $id_reservaciongeneral]);
        return $stmt->fetch(PDO::FETCH_ASSOC);
    }

    // Agregar una reserva
    public function agregarReserva($fecha, $id_guia, $id_agencia, $numero_pasajeros, $firma, $observaciones, $adelanto, $metodo_pago, $estado) {
        $stmt = $this->pdo->prepare("
            INSERT INTO reserva_general (fecha_mes_y_año, id_guia, id_agencia, numero_pasajeros, firma, observaciones, adelanto, metodo_pago, estado) 
            VALUES (:fecha, :id_guia, :id_agencia, :numero_pasajeros, :firma, :observaciones, :adelanto, :metodo_pago, :estado)
        ");
        $result = $stmt->execute([
            ":fecha" => $fecha,
            ":id_guia" => $id_guia,
            ":id_agencia" => $id_agencia,
            ":numero_pasajeros" => $numero_pasajeros,
            ":firma" => $firma ? htmlspecialchars($firma) : null,
            ":observaciones" => $observaciones ? htmlspecialchars($observaciones) : null,
            ":adelanto" => $adelanto !== null ? floatval($adelanto) : null,
            ":metodo_pago" => $metodo_pago ? htmlspecialchars($metodo_pago) : null,
            ":estado" => $estado
        ]);

        if ($result) {
            $id_reservaciongeneral = $this->pdo->lastInsertId();
            $reserva = $this->obtenerReservaPorId($id_reservaciongeneral);
            if ($reserva['precio_agencia'] !== null && $reserva['numero_pasajeros'] !== null && $reserva['adelanto'] !== null) {
                $totalidad = ($reserva['precio_agencia'] * $reserva['numero_pasajeros']) - $reserva['adelanto'];
                $totalidad = max(0, $totalidad);
                $this->actualizarTotalidad($id_reservaciongeneral, $totalidad);
            }
        }

        return $result;
    }

    // Editar una reserva
    public function editarReserva($id_reservaciongeneral, $fecha, $id_guia, $id_agencia, $numero_pasajeros, $firma, $observaciones, $adelanto, $metodo_pago, $estado) {
        try {
            $stmt = $this->pdo->prepare("
                UPDATE reserva_general 
                SET fecha_mes_y_año = :fecha, 
                    id_guia = :id_guia, 
                    id_agencia = :id_agencia, 
                    numero_pasajeros = :numero_pasajeros, 
                    firma = :firma, 
                    observaciones = :observaciones,
                    adelanto = :adelanto,
                    metodo_pago = :metodo_pago,
                    estado = :estado
                WHERE id_reservaciongeneral = :id_reservaciongeneral
            ");
            $result = $stmt->execute([
                ":fecha" => $fecha,
                ":id_guia" => $id_guia,
                ":id_agencia" => $id_agencia,
                ":numero_pasajeros" => $numero_pasajeros,
                ":firma" => $firma ? htmlspecialchars($firma) : null,
                ":observaciones" => $observaciones ? htmlspecialchars($observaciones) : null,
                ":adelanto" => $adelanto !== null ? floatval($adelanto) : null,
                ":metodo_pago" => $metodo_pago ? htmlspecialchars($metodo_pago) : null,
                ":estado" => $estado,
                ":id_reservaciongeneral" => $id_reservaciongeneral
            ]);

            if ($result) {
                $reserva = $this->obtenerReservaPorId($id_reservaciongeneral);
                if ($reserva['precio_agencia'] !== null && $reserva['numero_pasajeros'] !== null && $reserva['adelanto'] !== null) {
                    $totalidad = ($reserva['precio_agencia'] * $reserva['numero_pasajeros']) - $reserva['adelanto'];
                    $totalidad = max(0, $totalidad);
                    $this->actualizarTotalidad($id_reservaciongeneral, $totalidad);
                }
            }

            return $result;
        } catch (PDOException $e) {
            error_log("Error al editar reserva: " . $e->getMessage());
            return false;
        }
    }

    // Eliminar una reserva
    public function eliminarReserva($id_reservaciongeneral) {
        $stmt = $this->pdo->prepare("DELETE FROM reserva_general WHERE id_reservaciongeneral = :id_reservaciongeneral");
        return $stmt->execute([":id_reservaciongeneral" => $id_reservaciongeneral]);
    }
}