<?php
class EstadisticasService {
    private $reservas;

    public function __construct(array $reservas) {
        $this->reservas = $reservas;
    }

    public function calcularEstadisticasGenerales(): array {
        $totalReservas = count($this->reservas);
        $totalPasajeros = array_sum(array_column($this->reservas, 'numero_pasajeros'));
        $totalIngresos = array_sum(array_column($this->reservas, 'totalidad'));
        $promedioPasajeros = $totalReservas > 0 ? $totalPasajeros / $totalReservas : 0;

        return [
            'totalReservas' => $totalReservas,
            'totalPasajeros' => $totalPasajeros,
            'totalIngresos' => $totalIngresos,
            'promedioPasajeros' => $promedioPasajeros
        ];
    }

    public function calcularEstadisticasPorEstado(): array {
        $estados = ['pagado' => 0, 'sin pagar' => 0];
        $ingresosPorEstado = ['pagado' => 0, 'sin pagar' => 0];

        foreach ($this->reservas as $reserva) {
            $estado = $reserva['estado'] === 'pagado' ? 'pagado' : 'sin pagar';
            $estados[$estado]++;
            $ingresosPorEstado[$estado] += $reserva['totalidad'] ?? 0;
        }

        return [
            'estados' => $estados,
            'ingresosPorEstado' => $ingresosPorEstado
        ];
    }

    public function calcularEstadisticasPorMes(): array {
        $reservasPorMes = [];
        $ingresosPorMesPagado = [];
        $ingresosPorMesSinPagar = [];

        foreach ($this->reservas as $reserva) {
            $fecha = date('Y-m', strtotime($reserva['fecha_mes_y_año']));
            if (!isset($reservasPorMes[$fecha])) {
                $reservasPorMes[$fecha] = 0;
                $ingresosPorMesPagado[$fecha] = 0;
                $ingresosPorMesSinPagar[$fecha] = 0;
            }
            $reservasPorMes[$fecha]++;
            if ($reserva['estado'] === 'pagado') {
                $ingresosPorMesPagado[$fecha] += $reserva['totalidad'] ?? 0;
            } else {
                $ingresosPorMesSinPagar[$fecha] += $reserva['totalidad'] ?? 0;
            }
        }

        ksort($reservasPorMes);
        ksort($ingresosPorMesPagado);
        ksort($ingresosPorMesSinPagar);

        return [
            'reservasPorMes' => $reservasPorMes,
            'ingresosPorMesPagado' => $ingresosPorMesPagado,
            'ingresosPorMesSinPagar' => $ingresosPorMesSinPagar
        ];
    }

    public function calcularReservasPorGuia(): array {
        $reservasPorGuia = [];
        foreach ($this->reservas as $reserva) {
            $guia = $reserva['nombreGuia'];
            $reservasPorGuia[$guia] = ($reservasPorGuia[$guia] ?? 0) + 1;
        }
        return $reservasPorGuia;
    }

    public function calcularReservasPorAgencia(): array {
        $reservasPorAgencia = [];
        foreach ($this->reservas as $reserva) {
            $agencia = $reserva['nombreAgencia'];
            $reservasPorAgencia[$agencia] = ($reservasPorAgencia[$agencia] ?? 0) + 1;
        }
        return $reservasPorAgencia;
    }

    public function calcularMetodosPago(): array {
        $metodosPago = ['Efectivo' => 0, 'Tarjeta' => 0, 'Transferencia' => 0];
        foreach ($this->reservas as $reserva) {
            if (isset($reserva['metodo_pago']) && isset($metodosPago[$reserva['metodo_pago']])) {
                $metodosPago[$reserva['metodo_pago']]++;
            }
        }
        return $metodosPago;
    }
        // En terrazaAndina2/services/EstadisticasService.php

    public function calcularIngresosPromedioPorReserva(): float {
        $generales = $this->calcularEstadisticasGenerales();
        return $generales['totalReservas'] > 0 ? $generales['totalIngresos'] / $generales['totalReservas'] : 0;
    }

    public function calcularTasaOcupacion(): float {
        $porEstado = $this->calcularEstadisticasPorEstado();
        $totalReservas = array_sum($porEstado['estados']);
        return $totalReservas > 0 ? ($porEstado['estados']['pagado'] / $totalReservas) * 100 : 0;
    }

    public function calcularReservasPorDiaSemana(): array {
        $reservasPorDiaSemana = [
            'Lunes' => 0, 'Martes' => 0, 'Miércoles' => 0, 'Jueves' => 0,
            'Viernes' => 0, 'Sábado' => 0, 'Domingo' => 0
        ];

        foreach ($this->reservas as $reserva) {
            $diaSemana = date('l', strtotime($reserva['fecha_mes_y_año'])); // Obtiene el día de la semana en inglés
            $diaSemanaES = [
                'Monday' => 'Lunes', 'Tuesday' => 'Martes', 'Wednesday' => 'Miércoles',
                'Thursday' => 'Jueves', 'Friday' => 'Viernes', 'Saturday' => 'Sábado',
                'Sunday' => 'Domingo'
            ][$diaSemana];
            $reservasPorDiaSemana[$diaSemanaES]++;
        }

        return $reservasPorDiaSemana;
    }

    public function exportarACSV(string $filename = 'estadisticas_reservaciones.csv') {
        $generales = $this->calcularEstadisticasGenerales();
        $porEstado = $this->calcularEstadisticasPorEstado();
        $porMes = $this->calcularEstadisticasPorMes();
        $porGuia = $this->calcularReservasPorGuia();
        $porAgencia = $this->calcularReservasPorAgencia();
        $metodosPago = $this->calcularMetodosPago();

        // Configurar cabeceras para la descarga del archivo
        header('Content-Type: text/csv; charset=utf-8');
        header('Content-Disposition: attachment; filename="' . $filename . '"');

        // Crear un archivo CSV
        $output = fopen('php://output', 'w');

        // Escribir las cabeceras del CSV
        fputcsv($output, ['Estadística', 'Valor']);

        // Escribir las estadísticas generales
        fputcsv($output, ['Total de Reservaciones', $generales['totalReservas']]);
        fputcsv($output, ['Total de Pasajeros', $generales['totalPasajeros']]);
        fputcsv($output, ['Ingresos Totales', '$' . number_format($generales['totalIngresos'], 2)]);
        fputcsv($output, ['Promedio de Pasajeros por Reservación', number_format($generales['promedioPasajeros'], 1)]);

        // Reservaciones por estado
        fputcsv($output, []);
        fputcsv($output, ['Reservaciones por Estado', '']);
        fputcsv($output, ['Pagado', $porEstado['estados']['pagado']]);
        fputcsv($output, ['Sin Pagar', $porEstado['estados']['sin pagar']]);

        // Ingresos por estado
        fputcsv($output, []);
        fputcsv($output, ['Ingresos por Estado', '']);
        fputcsv($output, ['Pagado', '$' . number_format($porEstado['ingresosPorEstado']['pagado'], 2)]);
        fputcsv($output, ['Sin Pagar', '$' . number_format($porEstado['ingresosPorEstado']['sin pagar'], 2)]);

        // Reservaciones por mes
        fputcsv($output, []);
        fputcsv($output, ['Reservaciones por Mes', '']);
        fputcsv($output, ['Mes', 'Reservaciones']);
        foreach ($porMes['reservasPorMes'] as $mes => $cantidad) {
            fputcsv($output, [$mes, $cantidad]);
        }

        // Ingresos por mes
        fputcsv($output, []);
        fputcsv($output, ['Ingresos por Mes', '']);
        fputcsv($output, ['Mes', 'Pagado', 'Sin Pagar']);
        foreach ($porMes['ingresosPorMesPagado'] as $mes => $ingresoPagado) {
            $ingresoSinPagar = $porMes['ingresosPorMesSinPagar'][$mes] ?? 0;
            fputcsv($output, [$mes, '$' . number_format($ingresoPagado, 2), '$' . number_format($ingresoSinPagar, 2)]);
        }

        // Reservaciones por guía
        fputcsv($output, []);
        fputcsv($output, ['Reservaciones por Guía', '']);
        fputcsv($output, ['Guía', 'Reservaciones']);
        foreach ($porGuia as $guia => $cantidad) {
            fputcsv($output, [$guia, $cantidad]);
        }

        // Reservaciones por agencia
        fputcsv($output, []);
        fputcsv($output, ['Reservaciones por Agencia', '']);
        fputcsv($output, ['Agencia', 'Reservaciones']);
        foreach ($porAgencia as $agencia => $cantidad) {
            fputcsv($output, [$agencia, $cantidad]);
        }

        // Métodos de pago
        fputcsv($output, []);
        fputcsv($output, ['Métodos de Pago', '']);
        fputcsv($output, ['Método', 'Cantidad']);
        foreach ($metodosPago as $metodo => $cantidad) {
            fputcsv($output, [$metodo, $cantidad]);
        }

        fclose($output);
        exit();
    }
}