<?php
// Sección de verificación
require_once '../../config/auth.php';
requireLogin();

// Habilitar la depuración para ver errores
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Incluir configuración
require_once '../../config/config.php';

// Incluir controladores
require_once '../../controllers/ListadoMercadoController.php';
require_once '../../controllers/ReservaController.php';

// Incluir la librería FPDF
require_once '../../libs/fpdf/fpdf.php';

// Crear instancias de los controladores
$controller = new ListadoMercadoController($pdo);
$reservaController = new ReservaController($pdo);

// Obtener la fecha seleccionada del filtro (si existe), o usar la fecha actual por defecto
$fecha_actual = isset($_GET['fecha']) && !empty($_GET['fecha']) ? $_GET['fecha'] : date('Y-m-d');

// Validar el formato de la fecha seleccionada (debe ser YYYY-MM-DD)
$date = DateTime::createFromFormat('Y-m-d', $fecha_actual);
if (!$date || $date->format('Y-m-d') !== $fecha_actual) {
    $fecha_actual = date('Y-m-d'); // Si la fecha no es válida, usar la fecha actual
}

// Obtener la cantidad de pasajeros para la fecha seleccionada
try {
    $data = $reservaController->listarReservasConRelaciones($fecha_actual);
    $reservas = $data['reservas'];

    // Depuración: Ver cuántas reservaciones se están trayendo
    error_log("Reservaciones obtenidas para $fecha_actual: " . print_r($reservas, true));

    // Sumar la cantidad de pasajeros solo para las reservaciones del día exacto
    $total_pasajeros = 0;
    foreach ($reservas as $reserva) {
        $fecha_reserva = date('Y-m-d', strtotime($reserva['fecha_mes_y_año']));
        if ($fecha_reserva === $fecha_actual) {
            $total_pasajeros += $reserva['numero_pasajeros'];
        }
    }
    error_log("Total de pasajeros calculado para $fecha_actual: " . $total_pasajeros);
} catch (Exception $e) {
    $total_pasajeros = 0;
    error_log("Error al obtener pasajeros: " . $e->getMessage());
}

// Procesar el formulario para generar el PDF y guardar el pedido
if ($_SERVER["REQUEST_METHOD"] == "POST" && isset($_POST["generar_pedido"])) {
    $productos_seleccionados = $_POST["productos_seleccionados"] ?? [];
    $cantidades = $_POST["cantidades"] ?? [];

    if (empty($productos_seleccionados)) {
        header("Location: ListaRapidaMercado.php?fecha=$fecha_actual&error=Selecciona al menos un producto.");
        exit();
    }

    // Preparar los datos para el pedido
    $pedido = [];
    $total_general = 0;
    $precio_ajustado_total = 0;
    foreach ($productos_seleccionados as $id_productos) {
        $id_productos = intval($id_productos);
        $cantidad = isset($cantidades[$id_productos]) ? intval($cantidades[$id_productos]) : 0;

        if ($cantidad <= 0) {
            header("Location: ListaRapidaMercado.php?fecha=$fecha_actual&error=La cantidad debe ser mayor que 0 para el producto con ID $id_productos.");
            exit();
        }

        // Buscar el producto para obtener más detalles
        foreach ($controller->listarProductos() as $producto) {
            if ($producto['id_productos'] == $id_productos) {
                $costo_total = $producto['precio_mercado'] * $cantidad;
                $costo_ajustado = $producto['precio_referencial'] * $cantidad;
                $pedido[] = [
                    'id_productos' => $id_productos,
                    'nombreProdu' => $producto['nombreProdu'],
                    'categoria' => $producto['categoria'] ?? 'Sin categoría',
                    'unidad' => $producto['unidad'] ?? 'N/A',
                    'precio_mercado' => $producto['precio_mercado'],
                    'precio_referencial' => $producto['precio_referencial'], // Guardamos el precio referencial actual
                    'cantidad' => $cantidad,
                    'costo_total' => $costo_total,
                    'costo_ajustado' => $costo_ajustado
                ];
                $total_general += $costo_total;
                $precio_ajustado_total += $costo_ajustado;
                break;
            }
        }
    }

    // Guardar el pedido en la base de datos usando ListadoMercadoController
    try {
        $controller->guardarListaMercado($fecha_actual, $total_general, $pedido);
    } catch (Exception $e) {
        header("Location: ListaRapidaMercado.php?fecha=$fecha_actual&error=Error al guardar el pedido: " . urlencode($e->getMessage()));
        exit();
    }

    // Generar el PDF
    $pdf = new FPDF();
    $pdf->AddPage();
    $pdf->SetFont('Arial', 'B', 16);

    // Encabezado del PDF
    $pdf->Cell(0, 10, 'Lista Rapida de Mercado', 0, 1, 'C');
    $pdf->SetFont('Arial', '', 12);
    $pdf->Cell(0, 10, 'Fecha: ' . date('d/m/Y', strtotime($fecha_actual)), 0, 1, 'C');
    $pdf->Ln(10);

    // Tabla de productos
    $pdf->SetFont('Arial', 'B', 10);
    $pdf->SetFillColor(200, 220, 255);
    $pdf->Cell(30, 10, 'Producto', 1, 0, 'C', true);
    $pdf->Cell(25, 10, 'Categoria', 1, 0, 'C', true);
    $pdf->Cell(20, 10, 'Unidad', 1, 0, 'C', true);
    $pdf->Cell(20, 10, 'Cantidad', 1, 0, 'C', true);
    $pdf->Cell(25, 10, 'Precio Mercado', 1, 0, 'C', true);
    $pdf->Cell(25, 10, 'Precio Referencial', 1, 0, 'C', true);
    $pdf->Cell(25, 10, 'Costo Total', 1, 1, 'C', true);

    $pdf->SetFont('Arial', '', 10);
    foreach ($pedido as $item) {
        $pdf->Cell(30, 10, $item['nombreProdu'], 1);
        $pdf->Cell(25, 10, $item['categoria'], 1);
        $pdf->Cell(20, 10, $item['unidad'], 1);
        $pdf->Cell(20, 10, $item['cantidad'], 1, 0, 'C');
        $pdf->Cell(25, 10, 'S/ ' . number_format($item['precio_mercado'], 2), 1, 0, 'R');
        $pdf->Cell(25, 10, 'S/ ' . number_format($item['precio_referencial'], 2), 1, 0, 'R');
        $pdf->Cell(25, 10, 'S/ ' . number_format($item['costo_total'], 2), 1, 1, 'R');
    }

    // Total general
    $pdf->SetFont('Arial', 'B', 12);
    $pdf->Cell(145, 10, 'Total General:', 0, 0, 'R');
    $pdf->Cell(25, 10, 'S/ ' . number_format($total_general, 2), 0, 1, 'R');

    // Precio ajustado total
    $pdf->Cell(145, 10, 'Precio Ajustado Total:', 0, 0, 'R');
    $pdf->Cell(25, 10, 'S/ ' . number_format($precio_ajustado_total, 2), 0, 1, 'R');

    // Salida del PDF (mostrar en el navegador)
    $pdf->Output('I', 'Lista_Rapida_Mercado_' . date('Ymd', strtotime($fecha_actual)) . '.pdf');
    exit();
}

// Obtener datos para mostrar
$productos = $controller->listarProductos();

// Obtener todas las categorías únicas para el filtro
$categorias = array_unique(array_column($productos, 'categoria'));
sort($categorias);

// Filtrar productos por categoría si se selecciona una
$categoria_seleccionada = $_GET['categoria'] ?? '';
$productos_filtrados = $categoria_seleccionada ? array_filter($productos, function($producto) use ($categoria_seleccionada) {
    return ($producto['categoria'] ?? 'Sin categoría') === $categoria_seleccionada;
}) : $productos;

// Manejar mensajes de éxito o error
$success = $_GET['success'] ?? '';
$error = $_GET['error'] ?? '';
?>

<!DOCTYPE html>
<html lang="es">
<head>
    <meta charset="UTF-8">
    <title>Lista Rápida de Mercado</title>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
    <link rel="stylesheet" href="../public/css/sidebar.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.5.1/css/all.min.css" integrity="sha512-DTOQO9RWCH3ppGqcWaEA1BIZOC6xxalwEsw9c2QQeAIftl+Vegovlnee1c9QX4TctnWMn13TZye+giMm8e2LwA==" crossorigin="anonymous" referrerpolicy="no-referrer" />
    <!-- Incluir DataTables y el plugin rowGroup -->
    <link rel="stylesheet" href="https://cdn.datatables.net/1.13.6/css/dataTables.bootstrap5.min.css">
    <link rel="stylesheet" href="https://cdn.datatables.net/rowgroup/1.5.0/css/rowGroup.dataTables.min.css">
    <style>
        body {
            background-color: #f4f6f9;
        }
        .content {
            max-width: 1400px;
            margin: 0 auto;
            padding: 20px;
        }
        .card {
            border-radius: 10px;
            box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
            margin-bottom: 15px;
        }
        .card-header {
            background-color: #007bff;
            color: white;
            border-radius: 10px 10px 0 0;
            padding: 10px 15px;
            display: flex;
            justify-content: space-between;
            align-items: center;
        }
        .card-body {
            padding: 15px;
        }
        .form-control, .form-select {
            border-radius: 5px;
            font-size: 0.9rem;
            padding: 0.25rem 0.5rem;
        }
        .form-label {
            font-size: 0.9rem;
            margin-bottom: 0.25rem;
        }
        .btn {
            border-radius: 5px;
            font-size: 0.9rem;
            padding: 0.25rem 0.75rem;
        }
        .table th, .table td {
            vertical-align: middle;
            font-size: 0.85rem;
            padding: 4px 8px;
        }
        .table th {
            background-color: #e9ecef;
            font-weight: 600;
        }
        .price-highlight {
            font-weight: 500;
            color: #28a745;
        }
        .modal-backdrop { z-index: 1040 !important; }
        .modal { z-index: 1050 !important; }
        .dt-rowgroup {
            background-color: #f8f9fa;
            font-weight: bold;
            font-size: 1.1rem;
            padding: 10px;
            border-top: 2px solid #dee2e6;
        }
        .cantidad-input {
            width: 80px;
        }
        .pasajeros-info {
            font-size: 1rem;
            color: #333;
            background-color: #e9ecef;
            padding: 10px;
            border-radius: 5px;
            margin-top: 15px;
        }
        .filtros-card {
            margin-bottom: 15px;
        }
    </style>
</head>
<body>
<?php include '../templates/header.php'; ?>
    <!-- Main Content -->
    <div class="content">
        <h2 class="mb-3"><i class="fas fa-shopping-cart me-2"></i> Lista Rápida de Mercado</h2>

        <!-- Filtros combinados en un solo card -->
        <div class="card filtros-card">
            <div class="card-header">
                <h5 class="mb-0"><i class="fas fa-filter me-2"></i> Filtros</h5>
            </div>
            <div class="card-body">
                <form action="ListaRapidaMercado.php" method="GET" class="row g-2 align-items-end">
                    <div class="col-md-4">
                        <label for="fecha" class="form-label">Seleccionar Fecha:</label>
                        <input type="date" name="fecha" id="fecha" class="form-control" value="<?php echo htmlspecialchars($fecha_actual); ?>" onchange="this.form.submit()">
                    </div>
                    <div class="col-md-4">
                        <label for="categoria" class="form-label">Categoría:</label>
                        <select name="categoria" id="categoria" class="form-control" onchange="this.form.submit()">
                            <option value="">Todas las categorías</option>
                            <?php foreach ($categorias as $categoria): ?>
                                <option value="<?php echo htmlspecialchars($categoria); ?>" <?php if ($categoria_seleccionada === $categoria) echo 'selected'; ?>>
                                    <?php echo htmlspecialchars($categoria); ?>
                                </option>
                            <?php endforeach; ?>
                            <option value="Sin categoría" <?php if ($categoria_seleccionada === 'Sin categoría') echo 'selected'; ?>>Sin categoría</option>
                        </select>
                    </div>
                </form>
                <!-- Mostrar la cantidad de pasajeros para la fecha seleccionada -->
                <div class="pasajeros-info">
                    <i class="fas fa-users me-2"></i>
                    Cantidad de pasajeros para el <?php echo date('d/m/Y', strtotime($fecha_actual)); ?>: <strong><?php echo $total_pasajeros; ?></strong>
                </div>
            </div>
        </div>

        <!-- Formulario para generar el pedido -->
        <form action="ListaRapidaMercado.php?fecha=<?php echo htmlspecialchars($fecha_actual); ?>" method="POST">
            <!-- Mostrar todos los productos -->
            <div class="card">
                <div class="card-header">
                    <h5 class="mb-0"><i class="fas fa-table me-2"></i> Productos Disponibles</h5>
                    <button type="submit" name="generar_pedido" class="btn btn-primary btn-sm"><i class="fas fa-file-pdf me-2"></i> Generar Pedido</button>
                </div>
                <div class="card-body">
                    <table id="productosTable" class="table table-striped">
                        <thead>
                            <tr>
                                <th>Producto</th>
                                <th>Categoría</th>
                                <th>Unidad</th>
                                <th>Precio Abarrote</th>
                                <th>Precio Mercado</th>
                                <th>Cantidad</th>
                                <th>Seleccionar</th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php foreach ($productos_filtrados as $producto): ?>
                                <tr>
                                    <td><?php echo htmlspecialchars($producto['nombreProdu']); ?></td>
                                    <td><?php echo htmlspecialchars($producto['categoria'] ?? 'Sin categoría'); ?></td>
                                    <td><?php echo htmlspecialchars($producto['unidad'] ?? 'N/A'); ?></td>
                                    <td class="price-highlight">S/ <?php echo number_format($producto['precio_abarrote'], 2); ?></td>
                                    <td class="price-highlight">S/ <?php echo number_format($producto['precio_mercado'], 2); ?></td>
                                    <td>
                                        <input type="number" name="cantidades[<?php echo $producto['id_productos']; ?>]" class="form-control cantidad-input" min="1" value="1">
                                    </td>
                                    <td>
                                        <input type="checkbox" name="productos_seleccionados[]" value="<?php echo $producto['id_productos']; ?>" class="producto-checkbox">
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        </tbody>
                    </table>
                </div>
            </div>
        </form>

        <!-- Modal para Mensajes de Éxito -->
        <?php if ($success): ?>
            <div class="modal fade show" id="successModal" tabindex="-1" aria-labelledby="successModalLabel" aria-hidden="true" style="display: block;">
                <div class="modal-dialog">
                    <div class="modal-content">
                        <div class="modal-header">
                            <h5 class="modal-title" id="successModalLabel">Éxito</h5>
                            <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close" onclick="window.location.href='ListaRapidaMercado.php?fecha=<?php echo htmlspecialchars($fecha_actual); ?>';"></button>
                        </div>
                        <div class="modal-body">
                            <?php echo htmlspecialchars($success); ?>
                        </div>
                        <div class="modal-footer">
                            <button type="button" class="btn btn-primary" data-bs-dismiss="modal" onclick="window.location.href='ListaRapidaMercado.php?fecha=<?php echo htmlspecialchars($fecha_actual); ?>';">Cerrar</button>
                        </div>
                    </div>
                </div>
            </div>
            <div class="modal-backdrop fade show"></div>
        <?php endif; ?>

        <!-- Modal para Mensajes de Error -->
        <?php if ($error): ?>
            <div class="modal fade show" id="errorModal" tabindex="-1" aria-labelledby="errorModalLabel" aria-hidden="true" style="display: block;">
                <div class="modal-dialog">
                    <div class="modal-content">
                        <div class="modal-header">
                            <h5 class="modal-title" id="errorModalLabel">Error</h5>
                            <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close" onclick="window.location.href='ListaRapidaMercado.php?fecha=<?php echo htmlspecialchars($fecha_actual); ?>';"></button>
                        </div>
                        <div class="modal-body">
                            <?php echo htmlspecialchars($error); ?>
                        </div>
                        <div class="modal-footer">
                            <button type="button" class="btn btn-danger" data-bs-dismiss="modal" onclick="window.location.href='ListaRapidaMercado.php?fecha=<?php echo htmlspecialchars($fecha_actual); ?>';">Cerrar</button>
                        </div>
                    </div>
                </div>
            </div>
            <div class="modal-backdrop fade show"></div>
        <?php endif; ?>
    </div>

    <!-- Scripts -->
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js"></script>
    <!-- Incluir jQuery, DataTables y el plugin rowGroup -->
    <script src="https://code.jquery.com/jquery-3.6.0.min.js"></script>
    <script src="https://cdn.datatables.net/1.13.6/js/jquery.dataTables.min.js"></script>
    <script src="https://cdn.datatables.net/1.13.6/js/dataTables.bootstrap5.min.js"></script>
    <script src="https://cdn.datatables.net/rowgroup/1.5.0/js/dataTables.rowGroup.min.js"></script>
    <script>
        $(document).ready(function() {
            // Inicializar DataTables con rowGroup para productos
            $('#productosTable').DataTable({
                "language": {
                    "url": "//cdn.datatables.net/plug-ins/1.13.6/i18n/es-ES.json"
                },
                "pageLength": 10,
                "order": [[1, "asc"], [0, "asc"]], // Ordenar por categoría y luego por producto
                "searching": true,
                "paging": true,
                "info": true,
                "columnDefs": [
                    { "orderable": false, "targets": [5, 6] } // Deshabilitar ordenamiento en las columnas de cantidad y checkbox
                ],
                "rowGroup": {
                    "dataSrc": 1 // Agrupar por la columna Categoría (índice 1)
                }
            });
        });
    </script>
</body>
</html>