<?php
// Incluir la utilidad de autenticación
require_once '../../config/auth.php';
requireLogin();

// Configuración de depuración (mover a config.php en producción)
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Incluir dependencias
require_once '../../config/config.php';
require_once '../../controllers/ReservaController.php';

// Instancia del controlador
$controller = new ReservaController($pdo);

// Obtener parámetros de filtro
$estadoFiltro = isset($_GET['estado']) ? trim($_GET['estado']) : null;

// Obtener todas las reservas (con filtro por estado si aplica)
$data = $controller->listarReservasConRelaciones(null, null, null, null);
$reservas = $data['reservas'];

// Filtrar reservas por estado si se seleccionó un filtro
if ($estadoFiltro) {
    $reservas = array_filter($reservas, function($reserva) use ($estadoFiltro) {
        return $reserva['estado'] === $estadoFiltro;
    });
}

// Procesar reservas por fecha para el calendario
$eventosPorFecha = [];
foreach ($reservas as $reserva) {
    $fecha = $reserva['fecha_mes_y_año']; // Ejemplo: "2025-03-27"
    if (!isset($eventosPorFecha[$fecha])) {
        $eventosPorFecha[$fecha] = [];
    }
    $eventosPorFecha[$fecha][] = $reserva;
}

// Obtener mes, año y semana actuales o seleccionados
$mes = isset($_GET['mes']) ? intval($_GET['mes']) : date('m');
$anio = isset($_GET['anio']) ? intval($_GET['anio']) : date('Y');
$vista = isset($_GET['vista']) ? $_GET['vista'] : 'mensual'; // Vista por defecto: mensual
$semana = isset($_GET['semana']) ? intval($_GET['semana']) : date('W'); // Semana actual

// Calcular días del mes o semana
$primerDiaMes = new DateTime("$anio-$mes-01");
$ultimoDiaMes = (clone $primerDiaMes)->modify('last day of this month');
$diasEnMes = $ultimoDiaMes->format('d');
$primerDiaSemanaMes = $primerDiaMes->format('N') - 1; // 0 (lunes) a 6 (domingo)

// Configuración para la vista semanal
if ($vista === 'semanal') {
    $fechaInicioSemana = new DateTime();
    $fechaInicioSemana->setISODate($anio, $semana);
    $fechaInicioSemana->modify('monday this week'); // Comenzar en lunes
    $fechaFinSemana = (clone $fechaInicioSemana)->modify('+6 days');
    $semanaAnterior = $semana - 1;
    $semanaSiguiente = $semana + 1;
    $anioAnterior = $semana == 1 ? $anio - 1 : $anio;
    $anioSiguiente = $semana == 52 ? $anio + 1 : $anio;
    $semanaAnterior = $semana == 1 ? 52 : $semanaAnterior;
    $semanaSiguiente = $semana == 52 ? 1 : $semanaSiguiente;
} else {
    $mesAnterior = $mes == 1 ? 12 : $mes - 1;
    $anioAnterior = $mes == 1 ? $anio - 1 : $anio;
    $mesSiguiente = $mes == 12 ? 1 : $mes + 1;
    $anioSiguiente = $mes == 12 ? $anio + 1 : $anio;
}

// Día actual
$diaActual = date('Y-m-d'); // Ejemplo: "2025-03-27"
$mesActual = date('m');
$anioActual = date('Y');
$semanaActual = date('W');

// Obtener feriados dinámicamente desde la base de datos
$feriados = [];
try {
    $stmt = $pdo->prepare("SELECT fecha, nombre FROM feriados WHERE YEAR(fecha) = :anio");
    $stmt->execute(['anio' => $anio]);
    $feriadosData = $stmt->fetchAll(PDO::FETCH_ASSOC);
    foreach ($feriadosData as $feriado) {
        $fechaFeriado = $feriado['fecha'];
        $feriados[$fechaFeriado] = $feriado['nombre'];
    }
} catch (PDOException $e) {
    // Manejo de errores (puedes ajustar según tus necesidades)
    error_log("Error al obtener feriados: " . $e->getMessage());
    $feriados = [];
}
?>

<!DOCTYPE html>
<html lang="es">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Calendario de Reservas</title>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
    <link href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0-beta3/css/all.min.css" rel="stylesheet">
    <style>
        body {
            background-color: #f4f7fa;
            font-family: 'Poppins', sans-serif;
            color: #333;
        }
        .content {
            max-width: 1300px;
            margin: 0 auto;
            padding: 30px;
        }
        .card {
            background: #fff;
            border: none;
            border-radius: 12px;
            box-shadow: 0 6px 12px rgba(0, 0, 0, 0.1);
            padding: 25px;
        }
        h1 {
            font-size: 1.8rem;
            font-weight: 600;
            color: #1a73e8;
        }
        .calendar-header {
            display: flex;
            justify-content: space-between;
            align-items: center;
            margin-bottom: 15px;
        }
        .calendar-header a {
            font-size: 1.1rem;
            color: #1a73e8;
            text-decoration: none;
        }
        .calendar-header a:hover {
            color: #1557b0;
        }
        .calendar {
            display: grid;
            grid-template-columns: repeat(7, 1fr);
            gap: 3px;
            text-align: center;
        }
        .calendar.semanal {
            grid-template-columns: repeat(7, 1fr);
        }
        .calendar-day {
            padding: 8px;
            background: #f8f9fa;
            border-radius: 6px;
            min-height: 60px;
            position: relative;
            transition: background 0.3s ease;
            font-size: 0.85rem;
        }
        .calendar-day.header {
            background: #1a73e8;
            color: #fff;
            font-weight: 500;
            padding: 8px;
            font-size: 0.9rem;
        }
        .calendar-day.empty {
            background: #e9ecef;
        }
        .calendar-day.has-event-confirmed {
            background: #ff9999; /* Rojo pastel para Confirmado */
            cursor: pointer;
        }
        .calendar-day.has-event-confirmed:hover {
            background: #ff8080; /* Rojo pastel más oscuro para hover */
        }
        .calendar-day.has-event-pending {
            background: #ffd699; /* Amarillo pastel para Pendiente */
            cursor: pointer;
        }
        .calendar-day.has-event-pending:hover {
            background: #ffcc80; /* Amarillo pastel más oscuro para hover */
        }
        .calendar-day.high-load {
            background: #ff6666; /* Fondo más oscuro para alta carga */
        }
        .calendar-day.high-load:hover {
            background: #ff4d4d; /* Hover más oscuro para alta carga */
        }
        .calendar-day.today {
            border: 2px solid #1a73e8; /* Borde azul para el día actual */
            background: #e3f2fd; /* Fondo azul claro */
        }
        .calendar-day.holiday {
            position: relative;
        }
        .calendar-day.non-working {
            background: #d3d3d3; /* Gris claro para días no laborables */
        }
        .holiday-icon {
            position: absolute;
            top: 5px;
            left: 5px;
            color: #fbbc05; /* Amarillo para feriados */
            font-size: 0.9rem;
        }
        .event-dot {
            position: absolute;
            bottom: 3px;
            left: 50%;
            transform: translateX(-50%);
            width: 6px;
            height: 6px;
            border-radius: 50%;
        }
        .event-dot.confirmed {
            background: #ff9999; /* Rojo pastel para Confirmado */
        }
        .event-dot.pending {
            background: #ffd699; /* Amarillo pastel para Pendiente */
        }
        .event-dot.high-load {
            background: #ff6666; /* Más oscuro para alta carga */
        }
        .event-count {
            position: absolute;
            top: 5px;
            right: 5px;
            background: #1a73e8;
            color: #fff;
            font-size: 0.7rem;
            padding: 2px 5px;
            border-radius: 10px;
        }
        .modal-content {
            border-radius: 12px;
            box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15);
        }
        .modal-header {
            background: #1a73e8;
            color: #fff;
            border-top-left-radius: 12px;
            border-top-right-radius: 12px;
        }
        .modal-body p {
            margin: 5px 0;
            font-size: 0.9rem;
        }
        .btn-primary {
            background-color: #1a73e8;
            border-color: #1a73e8;
            border-radius: 8px;
            transition: background-color 0.3s ease;
        }
        .btn-primary:hover {
            background-color: #1557b0;
            border-color: #1557b0;
        }
        .btn-sm {
            font-size: 0.85rem;
            padding: 0.3rem 0.9rem;
        }
        .header-section {
            display: flex;
            justify-content: space-between;
            align-items: center;
            margin-bottom: 20px;
        }
        .view-toggle {
            display: flex;
            gap: 10px;
        }
        .filter-form {
            margin-bottom: 20px;
        }
        .filter-form .form-select {
            font-size: 0.9rem;
            border-radius: 8px;
            border: 1px solid #ced4da;
            box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.05);
        }
        .tooltip-custom {
            position: absolute;
            background: #333;
            color: #fff;
            padding: 5px 10px;
            border-radius: 4px;
            font-size: 0.8rem;
            z-index: 1000;
            pointer-events: none;
            white-space: pre-wrap;
            max-width: 200px;
        }
        footer {
            text-align: center;
            padding: 15px 0;
            background-color: #fff;
            box-shadow: 0 -2px 8px rgba(0, 0, 0, 0.05);
            position: fixed;
            bottom: 0;
            width: 100%;
            color: #666;
        }
        @media (max-width: 768px) {
            .calendar-day {
                padding: 5px;
                min-height: 50px;
                font-size: 0.75rem;
            }
            .calendar-day.header {
                font-size: 0.8rem;
            }
            .event-dot {
                width: 5px;
                height: 5px;
            }
            .event-count {
                font-size: 0.6rem;
                padding: 1px 4px;
            }
            .holiday-icon {
                font-size: 0.8rem;
            }
            .content {
                padding: 15px;
            }
            .header-section {
                flex-direction: column;
                align-items: flex-start;
                gap: 10px;
            }
            .view-toggle {
                flex-direction: column;
                gap: 5px;
            }
        }
    </style>
</head>
<body>
    <?php include '../templates/header.php'; ?>

    <div class="content">
        <div class="card">
            <!-- Sección de título, botón y cambio de vista -->
            <div class="header-section">
                <h1 class="mb-0">Calendario de Reservas</h1>
                <div class="view-toggle">
                    <a href="?mes=<?= $mes ?>&anio=<?= $anio ?>&vista=<?= $vista === 'mensual' ? 'semanal' : 'mensual' ?>&semana=<?= $semana ?>&estado=<?= $estadoFiltro ?>" class="btn btn-primary btn-sm">
                        Vista <?= $vista === 'mensual' ? 'Semanal' : 'Mensual' ?>
                    </a>
                    <a href="?mes=<?= $mesActual ?>&anio=<?= $anioActual ?>&vista=<?= $vista ?>&semana=<?= $semanaActual ?>&estado=<?= $estadoFiltro ?>" class="btn btn-primary btn-sm">
                        Ir al Día Actual
                    </a>
                    <a href="<?= BASE_URL ?>reserva/reservacreate.php" class="btn btn-primary btn-sm">Agregar Reservación</a>
                </div>
            </div>



            <!-- Navegación del calendario -->
            <div class="calendar-header">
                <?php if ($vista === 'mensual'): ?>
                    <a href="?mes=<?= $mesAnterior ?>&anio=<?= $anioAnterior ?>&vista=mensual&estado=<?= $estadoFiltro ?>">← Mes Anterior</a>
                    <h2 class="mb-0"><?= $primerDiaMes->format('F Y') ?></h2>
                    <a href="?mes=<?= $mesSiguiente ?>&anio=<?= $anioSiguiente ?>&vista=mensual&estado=<?= $estadoFiltro ?>">Mes Siguiente →</a>
                <?php else: ?>
                    <a href="?semana=<?= $semanaAnterior ?>&anio=<?= $anioAnterior ?>&vista=semanal&estado=<?= $estadoFiltro ?>">← Semana Anterior</a>
                    <h2 class="mb-0"><?= $fechaInicioSemana->format('d M') ?> - <?= $fechaFinSemana->format('d M Y') ?></h2>
                    <a href="?semana=<?= $semanaSiguiente ?>&anio=<?= $anioSiguiente ?>&vista=semanal&estado=<?= $estadoFiltro ?>">Semana Siguiente →</a>
                <?php endif; ?>
            </div>

            <!-- Calendario -->
            <div class="calendar <?= $vista === 'semanal' ? 'semanal' : '' ?>">
                <!-- Días de la semana -->
                <?php
                $diasSemana = ['Lun', 'Mar', 'Mié', 'Jue', 'Vie', 'Sáb', 'Dom'];
                foreach ($diasSemana as $dia) {
                    echo "<div class='calendar-day header'>$dia</div>";
                }

                if ($vista === 'mensual') {
                    // Espacios vacíos antes del primer día
                    for ($i = 0; $i < $primerDiaSemanaMes; $i++) {
                        echo "<div class='calendar-day empty'></div>";
                    }

                    // Días del mes
                    for ($dia = 1; $dia <= $diasEnMes; $dia++) {
                        $fechaActual = sprintf("%d-%02d-%02d", $anio, $mes, $dia);
                        $fechaObj = new DateTime($fechaActual);
                        $esDomingo = $fechaObj->format('N') == 7; // Domingo
                        $tieneEvento = isset($eventosPorFecha[$fechaActual]);
                        $esHoy = $fechaActual === $diaActual;
                        $esFeriado = array_key_exists($fechaActual, $feriados);
                        $esAltaCarga = $tieneEvento && count($eventosPorFecha[$fechaActual]) > 5; // Más de 5 reservas
                        $clase = $esDomingo ? 'non-working' : '';
                        $clase .= $esHoy ? ' today' : '';
                        $clase .= $esFeriado ? ' holiday' : '';
                        if ($tieneEvento) {
                            $estado = $eventosPorFecha[$fechaActual][0]['estado']; // Tomamos el estado del primer evento
                            $clase .= $estado === 'Confirmado' ? ' has-event-confirmed' : ' has-event-pending';
                            $clase .= $esAltaCarga ? ' high-load' : '';
                        }
                        echo "<div class='calendar-day $clase' data-fecha='$fechaActual' data-feriado-name='" . ($esFeriado ? htmlspecialchars($feriados[$fechaActual]) : '') . "'>";
                        echo $dia;
                        if ($esFeriado) {
                            echo "<i class='fas fa-star holiday-icon'></i>";
                        }
                        if ($tieneEvento) {
                            $numEventos = count($eventosPorFecha[$fechaActual]);
                            $estado = $eventosPorFecha[$fechaActual][0]['estado'];
                            $dotClase = $esAltaCarga ? 'high-load' : ($estado === 'Confirmado' ? 'confirmed' : 'pending');
                            echo "<div class='event-dot $dotClase'></div>";
                            echo "<span class='event-count'>$numEventos</span>";
                        }
                        echo "</div>";
                    }
                } else {
                    // Vista semanal
                    $fechaIterada = clone $fechaInicioSemana;
                    for ($i = 0; $i < 7; $i++) {
                        $fechaActual = $fechaIterada->format('Y-m-d');
                        $fechaObj = new DateTime($fechaActual);
                        $esDomingo = $fechaObj->format('N') == 7; // Domingo
                        $tieneEvento = isset($eventosPorFecha[$fechaActual]);
                        $esHoy = $fechaActual === $diaActual;
                        $esFeriado = array_key_exists($fechaActual, $feriados);
                        $esAltaCarga = $tieneEvento && count($eventosPorFecha[$fechaActual]) > 5; // Más de 5 reservas
                        $clase = $esDomingo ? 'non-working' : '';
                        $clase .= $esHoy ? ' today' : '';
                        $clase .= $esFeriado ? ' holiday' : '';
                        if ($tieneEvento) {
                            $estado = $eventosPorFecha[$fechaActual][0]['estado'];
                            $clase .= $estado === 'Confirmado' ? ' has-event-confirmed' : ' has-event-pending';
                            $clase .= $esAltaCarga ? ' high-load' : '';
                        }
                        echo "<div class='calendar-day $clase' data-fecha='$fechaActual' data-feriado-name='" . ($esFeriado ? htmlspecialchars($feriados[$fechaActual]) : '') . "'>";
                        echo $fechaIterada->format('d');
                        if ($esFeriado) {
                            echo "<i class='fas fa-star holiday-icon'></i>";
                        }
                        if ($tieneEvento) {
                            $numEventos = count($eventosPorFecha[$fechaActual]);
                            $estado = $eventosPorFecha[$fechaActual][0]['estado'];
                            $dotClase = $esAltaCarga ? 'high-load' : ($estado === 'Confirmado' ? 'confirmed' : 'pending');
                            echo "<div class='event-dot $dotClase'></div>";
                            echo "<span class='event-count'>$numEventos</span>";
                        }
                        echo "</div>";
                        $fechaIterada->modify('+1 day');
                    }
                }
                ?>
            </div>
        </div>
    </div>

    <!-- Modal para detalles de reservas -->
    <div class="modal fade" id="eventModal" tabindex="-1" aria-labelledby="eventModalLabel" aria-hidden="true">
        <div class="modal-dialog">
            <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title" id="eventModalLabel">Detalles de la Reserva</h5>
                    <button type="button" class="btn-close btn-close-white" data-bs-dismiss="modal" aria-label="Close"></button>
                </div>
                <div class="modal-body" id="eventDetails">
                    <!-- Detalles se llenarán con JavaScript -->
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">Cerrar</button>
                </div>
            </div>
        </div>
    </div>

    <?php include '../templates/footer.php'; ?>

    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js"></script>
    <script>
        // Datos de reservas en JavaScript
        const eventosPorFecha = <?php echo json_encode($eventosPorFecha); ?>;

        // Mostrar detalles al hacer clic en un día con evento
        document.querySelectorAll('.calendar-day.has-event-confirmed, .calendar-day.has-event-pending').forEach(day => {
            day.addEventListener('click', () => {
                const fecha = day.dataset.fecha;
                const eventos = eventosPorFecha[fecha];
                const modalBody = document.getElementById('eventDetails');
                const modalTitle = document.getElementById('eventModalLabel');

                modalTitle.textContent = `Reservas para ${fecha}`;
                modalBody.innerHTML = '';

                eventos.forEach(evento => {
                    modalBody.innerHTML += `
                        <div class="mb-3">
                            <p><strong>ID:</strong> ${evento.id_reservaciongeneral}</p>
                            <p><strong>Guía:</strong> ${evento.nombreGuia}</p>
                            <p><strong>Agencia:</strong> ${evento.nombreAgencia} (${evento.precio_agencia || 'N/A'})</p>
                            <p><strong>Nº Pasajeros:</strong> ${evento.numero_pasajeros}</p>
                            <p><strong>Firma:</strong> ${evento.firma}</p>
                            <p><strong>Observaciones:</strong> ${evento.observaciones || 'N/A'}</p>
                            <p><strong>Adelanto:</strong> ${evento.adelanto || 'N/A'}</p>
                            <p><strong>Método de Pago:</strong> ${evento.metodo_pago || 'N/A'}</p>
                            <p><strong>Totalidad:</strong> ${evento.totalidad || 'N/A'}</p>
                            <p><strong>Estado:</strong> <span class="badge ${evento.estado === 'Confirmado' ? 'bg-success' : 'bg-warning'}">${evento.estado}</span></p>
                            <hr>
                        </div>
                    `;
                });

                const modal = new bootstrap.Modal(document.getElementById('eventModal'));
                modal.show();
            });

            // Mostrar tooltip al pasar el mouse
            day.addEventListener('mouseover', (e) => {
                const fecha = day.dataset.fecha;
                const feriadoName = day.dataset.feriadoName;
                const eventos = eventosPorFecha[fecha];
                let tooltipText = feriadoName ? `Feriado: ${feriadoName}\n` : '';
                if (eventos) {
                    tooltipText += `Reservas: ${eventos.length}\n`;
                    eventos.forEach((evento, index) => {
                        tooltipText += `Guía ${index + 1}: ${evento.nombreGuia}\n`;
                        tooltipText += `Agencia ${index + 1}: ${evento.nombreAgencia}\n`;
                    });
                }

                if (tooltipText) {
                    const tooltip = document.createElement('div');
                    tooltip.className = 'tooltip-custom';
                    tooltip.textContent = tooltipText;
                    document.body.appendChild(tooltip);

                    const rect = day.getBoundingClientRect();
                    tooltip.style.top = `${rect.top - tooltip.offsetHeight - 5}px`;
                    tooltip.style.left = `${rect.left + (rect.width - tooltip.offsetWidth) / 2}px`;
                }
            });

            day.addEventListener('mouseout', () => {
                const tooltip = document.querySelector('.tooltip-custom');
                if (tooltip) {
                    tooltip.remove();
                }
            });
        });
    </script>
</body>
</html>